﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading;
using System.Diagnostics;
using System.Threading.Tasks;

namespace LoadBalancing
{
    class Program
    {
        const int ITERS = 200000;

        static void Main(string[] args)
        {
            while (true)
            {
                var sw = Stopwatch.StartNew();
                MyParallelForDynamic(0, ITERS, Workload3);
                Console.WriteLine("MyParallelFor: {0}", sw.Elapsed);

                sw.Restart();
                Parallel.For(0, ITERS, Workload3);
                Console.WriteLine("  ParallelFor: {0}", sw.Elapsed);

                Console.WriteLine("---------------------------------");
            }
        }

        static void Workload1(int i)
        {
            for (int j = 0; j < 100000; j++) ;
        }

        static void Workload2(int i)
        {
            for (int j = 0; j < i; j++) ;
        }

        static bool[] s_data = new bool[ITERS];
        static void Workload3(int i)
        {
            for (int j = 0; j < 100000; j++) s_data[i] = !s_data[i];
        }

        static void MyParallelForStatic(int fromInclusive, int toExclusive, Action<int> body)
        {
            int numProcs = Environment.ProcessorCount;
            int range = toExclusive - fromInclusive;
            int chunk = range / numProcs;

            int remaining = numProcs;
            using (ManualResetEventSlim mre = new ManualResetEventSlim())
            {
                for (int p = 0; p < numProcs; p++)
                {
                    int start = fromInclusive + (p * chunk);
                    int end = p == numProcs - 1 ? toExclusive : start + chunk;
                    ThreadPool.QueueUserWorkItem(delegate
                    {
                        for (int i = start; i < end; i++)
                        {
                            body(i);
                        }
                        if (Interlocked.Decrement(ref remaining) == 0) mre.Set();
                    });
                }
                mre.Wait();
            }
        }

        static void MyParallelForDynamic(int fromInclusive, int toExclusive, Action<int> body)
        {
            int numProcs = Environment.ProcessorCount;
            int nextIndex = fromInclusive - 1;

            int remaining = numProcs;
            using (ManualResetEventSlim mre = new ManualResetEventSlim())
            {
                for (int p = 0; p < numProcs; p++)
                {
                    ThreadPool.QueueUserWorkItem(delegate
                    {
                        int i;
                        while((i = Interlocked.Increment(ref nextIndex)) < toExclusive)
                        {
                            body(i);
                        }
                        if (Interlocked.Decrement(ref remaining) == 0) mre.Set();
                    });
                }
                mre.Wait();
            }
        }
    }
}