﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.Text.RegularExpressions;
using System.Threading;
using System.Threading.Tasks;

class Program
{
    static void Main(string[] args)
    {
        var data = Enumerable.Range(0, 3000000).Select(i => "{A:" + i + ", B:{C : " + (i * i) + "}}").ToList();
        while (true)
        {
            Time("Serial    ", () => RegexSerial(data));
            Console.ReadLine();
            Time("Parallel1 ", () => RegexParallel(data));
            Console.ReadLine();
            Time("Parallel2 ", () => RegexParallelThreadLocal(data));
            Console.ReadLine();
            Console.WriteLine();
        }
    }

    const string PATTERN = @"\G-{0,1}(?:0|[1-9]\d*)(?:\.\d*){0,1}(?:[eE][+-]{0,1}\d+){0,1}";
    const RegexOptions OPTIONS = RegexOptions.Compiled;

    static void RegexSerial(List<string> data)
    {
        var r = new Regex(PATTERN, OPTIONS);
        foreach (var d in data) r.Match(d);
    }

    #region W/O TLS
    static void RegexParallel(List<string> data)
    {
        var r = new Regex(PATTERN, OPTIONS);
        Parallel.ForEach(data, d => r.Match(d));
    }
    #endregion

    #region TLS
    static void RegexParallelThreadLocal(List<string> data)
    {
        using (var r = new ThreadLocal<Regex>(() => new Regex(PATTERN, OPTIONS)))
            Parallel.ForEach(data, d => r.Value.Match(d));
    }
    #endregion

    static void Time(string name, Action body)
    {
        GC.Collect();
        var sw = Stopwatch.StartNew();
        body();
        Console.WriteLine("{0}: {1}", name, sw.Elapsed);
    }
}